DrDamage = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1")
--local L = AceLibrary("AceLocale-2.2"):new("DrDamage");
local SEA = AceLibrary("SpecialEvents-Aura-2.0")
local GT = AceLibrary("Gratuity-2.0")

local defaults = {
	SpellDamage = 0,
	ShadowDamage = 0,
	FireDamage = 0,
	NatureDamage = 0,
	FrostDamage = 0,
	ArcaneDamage = 0,
	ItemCrit = 0,
	ItemHit = 0,
	DisplayType = "AvgDMG",
	ManualDmg = false,
	ManualPlus = false,
	FontSize = 8,
	FontPosition = 0,
}

DrDamage:RegisterDB("DrDamageDB");
DrDamage:RegisterDefaults("profile", defaults )

function DrDamage:OnInitialize()	
end

local changesAB = {
    "ACTIONBAR_PAGE_CHANGED",
    "ACTIONBAR_SLOT_CHANGED",
    "UNIT_INVENTORY_CHANGED",
    "PLAYER_TARGET_CHANGED",
    --"PLAYER_REGEN_ENABLED",
   -- "PLAYER_AURAS_CHANGED",
    "UNIT_AURA",
}

local changesDMG = {
    "CHARACTER_POINTS_CHANGED",
    "LEARNED_SPELL_IN_TAB",
    "PLAYER_ALIVE",
}

function DrDamage:DrOnEvent()

	for i = 1, #changesAB do
    	
	    	if changesAB[i] and changesAB[i] == event then
	    	    	
	    		if event == "UNIT_AURA" then
	    		
	    			if arg1 == "target" or arg1 =="player" then
	    			
	    				if not self:IsEventScheduled("UpdatingAB") then
	    					self:ScheduleEvent("UpdatingAB", self.UpdateAB, 1.5, self)
	    				end
	    			end
	    		else
	    			if not self:IsEventScheduled("UpdatingAB") then
	    				self:ScheduleEvent("UpdatingAB", self.UpdateAB, 0.5, self)
	    			end
	    		end
	    	end
	end
	
	for i = 1, #changesDMG do
    	
    		if changesDMG[i] and changesDMG[i] == event then
    			DrDamage:DmgUpdateTalents()
    			DrDamage:UpdateAB()
    		end
    	end
end

function DrDamage:Compatible( check )

	if check == "Class" then
		localizedClass, englishClass = UnitClass("player");
		
		if englishClass == "WARLOCK" or englishClass =="MAGE" or englishClass == "PRIEST" then
			return true
		else
			return false
		end
	end
	
	return false
end
		  
function DrDamage:OnEnable()

	if DrDamage:Compatible( "Class" ) then
		DrDamage:Hook( GameTooltip, "SetAction", true)
		DrDamage:SecureHook( GameTooltip, "SetSpell" )
		DrDamage:DmgUpdateTalents()
	else
		self:Print( "Your class is not supported by Dr. Damage." )
		do return end
	end
	
    	for i = 1, #changesDMG do
    		
    		if changesDMG[i] then
    			self:RegisterEvent( changesDMG[i], "DrOnEvent" )
    		end
  	end
	
	if DrDamage.db.profile.DisplayType ~= "Off" then
    		
    		DrDamage:UpdateAB()
    	
    		for i = 1, #changesAB do
    	
    			if changesAB[i] then
    				self:RegisterEvent( changesAB[i], "DrOnEvent" )
    			end
    		end
  	end   
end

function DrDamage:OnDisable()

	DrDamage:UpdateAB( true )
	talents = nil
	
end

function DrDamage:SetSpell( frame, slot )

	GT:SetSpell( slot, BOOKTYPE_SPELL )
	local spellName, spellRank = GetSpellName( slot, BOOKTYPE_SPELL );
	spellRank = string.sub( spellRank, 6 )
	
	if not spellRank then
		spellRank = "1"
	end
	
	DrDamage:DoTooltip( frame, spellName, spellRank )

end

function DrDamage:SetAction( frame, slot )

	self.hooks[frame].SetAction(frame, slot)

	GT:SetAction( slot )
	local spellName = getglobal("GameTooltipTextLeft1"):GetText()
	--NEEDS LOCALIZATION
	local _, _, spellRank = GT:Find("Rank (%d+)", 1, 1, true)
	
	if not spellRank then
		spellRank = "1"
	end
	
	DrDamage:DoTooltip( frame, spellName, spellRank )
end

function DrDamage:DoTooltip( frame, spellName, spellRank )

	if spellName and spellRank then
  	
  		local spellInfo = DrDamage:DetermineClass( "Spells" )
  		
		if not spellInfo then
			do return end
		end  		
  	
		if spellInfo[spellName] then
	
			local spell = spellInfo[spellName]
			local tableSpell = spell[spellRank]
			
			if not tableSpell then
				tableSpell = spell["1"]
			end
					
			if tableSpell then
			
				--NEEDS LOCALIZATION
				local _, _, ManaCost = GT:Find("(%d+) Mana", 2, 2)
				
				if ManaCost then
					ManaCost = tonumber( ManaCost )
				else
					ManaCost = 0
				end
				
				local PlayerMana = UnitManaMax("player")
			
				local MinDmg, MaxDmg, AvgDmg, AvgDmgCrit, CritRate, DotDmg, DPS, DPSC, SpellDam, SpellM, DamM, NextCrit, NextTenDam = DrDamage:RawNumbers ( tableSpell, spellName )
				local DPM = DrDamage:Round( AvgDmgCrit / ManaCost, 1 )
				local DOOM = DrDamage:Round( DPM * PlayerMana, 1 )
				
				frame:AddLine(" ")
				frame:AddLine("Dr. Damage:", 1,1,1)
				frame:AddLine(" ")
				frame:AddDoubleLine("+Dmg: ", SpellDam .. " (x" .. SpellM .. ")", 1, 1, 1, 0, 0.7, 0  )
  				
  				if CritRate > 0 then
  					frame:AddDoubleLine("Crit:", CritRate .. "%", 1, 1, 1, 0, 0.7, 0 )
  				end	
  				
  				frame:AddDoubleLine("Avg (x".. DamM .. "):", AvgDmg .. " (".. MinDmg .."-".. MaxDmg ..")", 1, 1, 1, 0.8, 0.8, 0.9 )
  				
  				if AvgDmgCrit > AvgDmg then
  					frame:AddDoubleLine("Avg Crit/DoT:", AvgDmgCrit, 1, 1, 1, 0.8, 0.8, 0.9  )
  				end
  				
   				if DotDmg > 0 then
  					frame:AddDoubleLine("DoT:", DotDmg, 1, 1, 1, 0.8, 0.8, 0.9 )
  				end 				
  				
  				if NextCrit > 0 then
  					frame:AddDoubleLine("+1% Crit:", "+" .. NextCrit .. " (= " .. DrDamage:Round( ( NextCrit * 10 ) / NextTenDam, 1 ) .. " +dmg)", 1, 1, 1, 0.3, 0.6, 0.5 )	
  				end
  				
  				frame:AddDoubleLine( "+10 Dmg:", "+" .. NextTenDam, 1, 1, 1, 0.3, 0.6, 0.5 )
  				
  				
  				if DPSC ~= DPS then
  					frame:AddDoubleLine("DPS/DPSC:", DPS .. "/" .. DPSC, 1, 1, 1, 0.8, 0.1, 0.1)
      				else
      					frame:AddDoubleLine("DPS:", DPS, 1, 1, 1, 0.8, 0.1, 0.1)
      				end
      				
      				if ManaCost > 0 then
      					frame:AddDoubleLine("DPM:", DPM, 1, 1, 1, 0.8, 0.1, 0.1 )
      					frame:AddDoubleLine("Damage until OOM:", DOOM, 1, 1, 1, 0.8, 0.1, 0.1 )
      				end
      				
      				frame:Show()			
			end  			
		end	  
	end
end


function dGet (n)
	return function () return DrDamage.db.profile[n] 
	end
end
function dSet (n, v)
	return function (v) DrDamage.db.profile[n] = v; DrDamage:UpdateAB(); 
	end
end

local DisplayTypes = { "Avg", "DPS", "DPSC", "Off" }

local options = { 
	type='group',
	args = {
		ActionBar = {
			type = "group", desc = "Options for the actionbar", name="ActionBar",
			args = {
				DisplayType = {
					name = "Display",
					type = "text",
					desc = "Choose what to display on the actionbar",
					validate = DisplayTypes,
					get = dGet("DisplayType"),
					set = dSet("DisplayType", value),
					order = 70,
				},
        			Update = {
            				name = "Update",
            				desc = "Forces update to actionbar",
            				type = "execute",
            				order = 75,
            				func = "UpdateAB",
    				}, 
				FontSize = {
					type = 'range',
					name = "Actionbar font size",
					min = 6,
					max = 14,
					step = 1,
					desc = "Set the font size",
					order = 71,
					get = dGet("FontSize"),
					set = dSet("FontSize", value),
				},
				FontPosition = {
					type = 'range',
					name = "Actionbar font position",
					min = -5,
					max = 5,
					step = 1,
					desc = "Set the font position",
					order = 72,
					get = dGet("FontPosition"),
					set = dSet("FontPosition", value),
				},    				
			},
		},
		Damage = {
			type = "group", desc = "Damage options", name="Damage",
			args = {
				ManualDmg = {
					type = 'toggle',
					name = "Manual variables",
					desc = "Allows you to manually set damage properties",
					order = 71,
					get = dGet( "ManualDmg" ),
					set = dSet( "ManualDmg", value ),
				},
				ManualPlus = {
					type = 'toggle',
					name = "Manual damage adds to detected",
					desc = "Allows you to set if manual damage should add to detected damage",
					order = 72,
					get = dGet( "ManualPlus" ),
					set = dSet( "ManualPlus", value ),
				},   		
				SpellDamage = {
					type = 'range',
					name = "Spell Damage",
					min = 0,
					max = 2000,
					step = 1,
					desc = "Input your spell damage",
					order = 80,
					get = dGet("SpellDamage"),
					set = dSet("SpellDamage", value),
				},		
				FireDamage = {
					type = 'range',
					name = "Fire Damage",
					min = 0,
					max = 2000,
					step = 1,
					desc = "Input your fire +damage",
					order = 81,
					get = dGet("FireDamage"),
					set = dSet("FireDamage", value),		
		
				},
				ShadowDamage = {
					type = 'range',
					name = "Shadow Damage",
					min = 0,
					max = 2000,
					step = 1,
					desc = "Input your shadow +damage",
					order = 82,
					get = dGet("ShadowDamage"),
					set = dSet("ShadowDamage", value),		
				},
				FrostDamage = {
					type = 'range',
					name = "Frost Damage",
					min = 0,
					max = 2000,
					step = 1,
					desc = "Input your frost +damage",
					order = 83,
					get = dGet("FrostDamage"),
					set = dSet("FrostDamage", value),		
				},
				NatureDamage = {
					type = 'range',
					name = "Nature Damage",
					min = 0,
					max = 2000,
					step = 1,
					desc = "Input your nature +damage",
					order = 84,
					get = dGet("NatureDamage"),
					set = dSet("NatureDamage", value),		
				},
				ArcaneDamage = {
					type = 'range',
					name = "Arcane Damage",
					min = 0,
					max = 2000,
					step = 1,
					desc = "Input your arcane +damage",
					order = 85,
					get = dGet("ArcaneDamage"),
					set = dSet("ArcaneDamage", value),		
				},				
				ItemCrit = {
					type = 'range',
					name = "Item Crit Bonus",
					min = 0,
					max = 1,
					isPercent = true,
					desc = "Input your crit bonus from items",
					order = 88,
					get = dGet("ItemCrit"),
					set = dSet("ItemCrit", value),					
				},
				ItemHit = {
					type = 'range',
					name = "Item Hit Bonus",
					min = 0,
					max = 1,
					isPercent = true,
					desc = "Input your hit bonus from items",
					order = 89,
					get = dGet("ItemHit"),
					set = dSet("ItemHit", value),	
				},
			},
		},
	},
};

DrDamage:RegisterChatCommand( { "/drdmg", "/drdamage" }, options);

local talents = {};

function DrDamage:DmgUpdateTalents()

	talents = {}
	local talentList = DrDamage:DetermineClass( "Talents" )
	
	if not talentList then
		do return end
	end
	
	local numTabs = GetNumTalentTabs();
	
	for t=1, numTabs do
	
    		local numTalents = GetNumTalents(t);
    		  		
	    	for i=1, numTalents do
	    	
	    		nameTalent, _, _, _, currRank, maxRank= GetTalentInfo(t,i);
	    		
	    	    	if currRank ~= 0 then
	    	    			    	    
	    	    		if talentList[nameTalent] then
	    	    		
	    	    			local v = talentList[nameTalent]
	    	    			    	    			
	    	    			table.insert( talents, { Name = nameTalent, Value = currRank * v.Effect } )
	    	    			--self:Print("Talent name: " .. nameTalent .. " Points: " .. currRank .. "/" .. maxRank )
	    	    		end    	    	
	    	    	end	    	    
	    	end
	end
end


function DrDamage:DetermineClass( requestType )

	localizedClass, englishClass = UnitClass("player");
	local ClassTable
	
	if englishClass == "WARLOCK" then
		ClassTable = WarlockTable
	elseif englishClass == "MAGE" then
		ClassTable = MageTable
	elseif englishClass == "PRIEST" then
		ClassTable = PriestTable
	else
		self:Print( "Unsupported class detected." )
	end
	
	if ClassTable then
	
		if requestType == "Talents" then
			return ClassTable["talentInfo"]	
		elseif requestType == "Spells" then
			return ClassTable["spellInfo"]
		end
	end
end
	

function DrDamage:UpdateAB( disable )

	local blizzAction
	local BT3 = false
	local Bongos = false
	local DAB = false
	local NAB = false
	local blizzAB = false

	if IsAddOnLoaded("Bartender3") then
		BT3 = true
	elseif IsAddOnLoaded( "Bongos" ) then
		Bongos = true
	elseif IsAddOnLoaded("DiscordActionBars") then
		DAB = true
	elseif IsAddOnLoaded("Nurfed_ActionBars") then
		NAB = true
	else
		blizzAB = true
		blizzAction = { "ActionButton", "BonusActionButton", "MultiBarBottomLeftButton", "MultiBarBottomRightButton","MultiBarRightButton", "MultiBarLeftButton", }
	end
	
	local spellInfo = DrDamage:DetermineClass( "Spells" )
	
	if not spellInfo then
		do return end
	end
	
	for i = 1, 120 do
	
		local ab
		local name
	
		if BT3 then
			name = "BT3Button"..i
			ab = Bartender3.actionbuttons
		elseif Bongos then
			name = "BActionButton"..i
		elseif DAB then
			name = "DAB_ActionButton_"..i
		elseif NAB then
			name = "Nurfed_ActionButton"..i
		elseif blizzAB then
			local abType = 1 + math.floor( ( i - 1 ) / 12 )
			local buttonNr = i - math.floor( ( i - 1 ) / 12 ) * 12
			
			if abType > #blizzAction then
				do return end
			end
			
			name = blizzAction[abType] .. buttonNr
		end
					
		local button = getglobal(name)
					
		if button then
			local children = { button:GetChildren() }
		
			if children then
   				for i = 1, #children do
   				
   					local frame = children[i]
   		
   					if frame and frame.text and frame:GetID() == 200 then
   						frame.text:Hide()
   					end
   					
   				end
   			end			
		end
			
		if button and button:IsVisible() and not disable then
		
			local id
		
			if BT3 and ab[i] and ab[i].state == "used" then
				id = ab[i].object:PagedID()
			elseif Bongos then
				id = button:GetPagedID()
			elseif DAB then
				id = button:GetActionID()
			elseif NAB then
				id = button:GetID()
			elseif blizzAB then
				id = ActionButton_GetPagedID( button )
			end
			
			if id then
			
				local gtype, pid, gsubtype = GetActionInfo(id)
			
				if gtype == "spell" then
			
					local spellName, spellRank = GetSpellName( pid, BOOKTYPE_SPELL );
			
					if spellInfo[spellName] then

						local spell = spellInfo[spellName]
						local tableSpell = spell[string.sub( spellRank, 6 )]
						
						if not tableSpell then
							tableSpell = spell["1"]
						end
		
						if tableSpell then

							_, _, _, AvgDmgCrit, _, _, DPS, DPSC = DrDamage:RawNumbers ( tableSpell, spellName )
								
							if DrDamage.db.profile.DisplayType == "DPS" then
								DrDamage:CreateABtext( button, DPS )
							elseif DrDamage.db.profile.DisplayType == "DPSC" then
								DrDamage:CreateABtext( button, DPSC )
							elseif DrDamage.db.profile.DisplayType == "Avg" then
								DrDamage:CreateABtext( button, AvgDmgCrit )
							end
						end				
					end			
				end
			end
		end
	end	
end


function DrDamage:CreateABtext( button, cDmg )

	local children = { button:GetChildren() }
	local found = false
	
	if children then
   		for i = 1, #children do
   		
   			local frame = children[i]
   			
   			if frame and frame.text and frame:GetID() == 200 then
   				 				
   				frame.text:SetPoint("BOTTOMLEFT", frame, "BOTTOM", (DrDamage.db.profile.FontPosition - 15), 5)
   				frame.text:SetFont( STANDARD_TEXT_FONT, DrDamage.db.profile.FontSize, "OUTLINE" )
   				frame.text:SetText( cDmg )
   				frame.text:Show()
   				found = true
   			end
   		end
   	end
   		
   	if not found then
   		
		local dpsText = CreateFrame("Frame", "DrDamageText", button )
		dpsText:SetAllPoints( button )
		dpsText:SetFrameLevel(dpsText:GetFrameLevel() + 1)
		dpsText:SetID( 200 )
		dpsText.text = dpsText:CreateFontString( nil, "OVERLAY" )
		dpsText.text:SetFontObject(GameFontNormalSmall)
		dpsText.text:SetPoint("BOTTOMLEFT", dpsText, "BOTTOM", (DrDamage.db.profile.FontPosition - 15), 5)
		dpsText.text:SetFont(STANDARD_TEXT_FONT, DrDamage.db.profile.FontSize, "OUTLINE")
		dpsText.text:SetTextColor(1, 1, 0.2)
		dpsText.text:SetText( cDmg )
		dpsText.text:Show()
	end

end

function DrDamage:SpellCrit()

	if ManualDmg then
		local crit = GetSpellCritChanceFromIntellect() + DrDamage.db.profile.ItemCrit * 100
		return crit
	else
		local crit = GetSpellCritChanceFromIntellect() + GetCombatRatingBonus(11)
		return crit
	end
end

function DrDamage:GetPlusDamage( school )

	local spellDMG = GetSpellBonusDamage(1)
	
	if school == "Holy" then
  		return GetSpellBonusDamage(2)
	elseif school == "Fire" then
  		return GetSpellBonusDamage(3)
  	elseif school == "Nature" then
  		return GetSpellBonusDamage(4)
  	elseif school == "Frost" then
  		return GetSpellBonusDamage(5)
  	elseif school == "Shadow" then
  		return GetSpellBonusDamage(6)  		
   	elseif school == "Arcane" then
  		return GetSpellBonusDamage(7)
  	else
  		return spellDMG
  	end	
end

function DrDamage:RawNumbers( spell, spellName )

	if not spell or not spellName then
		do return end
	end
		
	local spellInfo = DrDamage:DetermineClass( "Spells" )
	
	if not spellInfo then
		do return end
	end	
	
	local baseSpell = (spellInfo[spellName])["0"]
	
	if not talents or #talents < 1 then
		DrDamage:DmgUpdateTalents()
	end

	local spellDmg = DrDamage:GetPlusDamage( baseSpell.School ) --+all spelldamage
	local critPerc = DrDamage:SpellCrit() 	--Calculate crit
	
	if DrDamage.db.profile.ManualDmg then
		
		if not DrDamage.db.profile.ManualPlus then
			spellDmg = DrDamage.db.profile.SpellDamage
			critPerc = DrDamage.db.profile.ItemCrit
		else
			critPerc = critPerc + DrDamage.db.profile.ItemCrit
		end
		
		if ( baseSpell.School == "Shadow" ) then
			spellDmg = spellDmg + DrDamage.db.profile.ShadowDamage --add shadowdamage
		elseif ( baseSpell.School == "Fire" ) then
			spellDmg = spellDmg + DrDamage.db.profile.FireDamage --add firedamage
		elseif ( baseSpell.School == "Nature" ) then
			spellDmg = spellDmg + DrDamage.db.profile.NatureDamage --add naturedamage
		elseif ( baseSpell.School == "Frost" ) then
			spellDmg = spellDmg + DrDamage.db.profile.FrostDamage --add frostdamage
		elseif ( baseSpell.School == "Arcane" ) then
			spellDmg = spellDmg + DrDamage.db.profile.ArcaneDamage --add arcanedamage			
		end
	end
			
	local critM = 0.5			--Crit multiplier
	local castTime = spell.castTime		--Spell's casttime
	local bNukeDmg = 0			--Modifier for spells with a DOT portion
	local dmgM = 1				--Damage multiplier (total)
	local bDmgM = 1				--Base damage Modifier
	local spellDmgM				--Spell damage multiplier
	local igniteM = 0

	
	if baseSpell.eDot then 
		spellDmgM = (spell.eDuration / 15)
	else
		spellDmgM = (spell.castTime / 3.5) 
	end
	
	if baseSpell.sFactor then
		spellDmgM = spellDmgM * baseSpell.sFactor
	elseif spell.sFactor then
		spellDmgM = spellDmgM * spell.sFactor
	end
	
	local talentEffects = DrDamage:DetermineClass( "Talents" )
	
	if not talentEffects then
		do return end
	end
			
	for i, charTalent in ipairs( talents ) do
		
		local talentName = charTalent.Name
		local talentValue = charTalent.Value
		
		if talentName then
					
			if talentEffects[talentName] then
				
				local effectTalent = talentEffects[talentName]
				
				function DoCalc( ModType, talentSpell )
				
					if talentSpell == spellName or talentSpell == "All" then
				
						if ModType == "SpellDamage" then
							spellDmgM = spellDmgM + talentValue
						elseif ModType == "SpellDamageBonus" then
							spellDmgM = spellDmgM * ( talentValue + 1 )
						elseif ModType == "CastTime" then
							
							if baseSpell.CastMod then
								castTime = castTime - baseSpell.CastMod * talentValue
							else
								castTime = castTime - talentValue
							end
								
							if castTime < 1.5 then
								castTime = 1.5
							end
						elseif ModType == "BaseNuke" then
							bNukeDmg = bNukeDmg + talentValue
						elseif ModType == "Crit" then
							critPerc = critPerc + talentValue
						elseif ModType == "CritMultiplier" then
							critM = talentValue
						elseif ModType == "Ignite" then
							igniteM = talentValue
						elseif not ModType then
							dmgM = dmgM + talentValue		
						end
					end				
				
				end
						
				for j, talentSpell in ipairs( effectTalent.Spells ) do
					DoCalc( effectTalent.ModType, talentSpell  )
				end
				
				if effectTalent.Spells2 then
					for j, talentSpell in ipairs( effectTalent.Spells2 ) do
						DoCalc( effectTalent.ModType2, talentSpell  )
					end
				end	
			end
		end		
	end
	
	--Start calculating Buffs/Debuffs that affects damage not provided by blizzard API (What is missing?)
	
	for buffName, index in SEA:DebuffIter("target") do

		if baseSpell.School == "Shadow" then
			
			if buffName == "Curse of Shadow" then
				dmgM = dmgM + 0.1
			elseif buffName == "Shadow Vulnerability" then
				local index, apps, texture, rank = SEA:UnitHasDebuff( "target", "Shadow Vulnerability" )
		
				if string.find( texture, "Spell_Shadow_ShadowBolt" ) then
					dmgM = dmgM + 0.2
				else
					dmgM = dmgM + 0.03 * apps
				end	
			elseif buffName == "Shadow Weakness" then
				dmgM = dmgM + 1
			end
		
		elseif baseSpell.School == "Fire" then
		
			if buffName == "Fire Vulnerability" then
				local index, apps, texture, rank = SEA:UnitHasDebuff( "target", "Fire Vulnerability" )
				dmgM = dmgM + apps * 0.03
			elseif buffName == "Curse of the Elements" then
				dmgM = dmgM + 0.1
			elseif buffName == "Fire Weakness" then
				dmgM = dmgM + 1
			end
			
		elseif baseSpell.School == "Frost" then 
		
			if buffName == "Winter's Chill" then
				local index, apps, texture, rank = SEA:UnitHasDebuff( "target", "Winter's Chill" )
				critPerc = critPerc + apps * 2
			elseif buffName == "Curse of the Elements" then
				dmgM = dmgM + 0.1
			elseif buffName == "Frost Weakness" then
				dmgM = dmgM + 1
			end
		elseif baseSpell.School == "Arcane" then
		
			if buffName == "Curse of Shadow" then
				dmgM = dmgM + 0.1
			elseif buffName == "Arcane Weakness" then
					dmgM = dmgM + 1
			end	
		end
		
		if buffName == "Spell Vulnerability" then
			dmgM = dmgM + 0.15
		elseif buffName == "Misery" then
			--Does this work? Does misery have a rank returned?
			local index, apps, texture, rank = SEA:UnitHasDebuff( "target", "Misery" )
			dmgM = dmgM + rank * 0.01
		elseif buffName == "Berserking" then
			dmgM = dmgM + 0.1
		end
	end
	
	for buffName, index in SEA:BuffIter("player") do
  		
 		if baseSpell.School == "Fire" then
 			if buffName == "Burning Wish" then
				local index, apps, texture, rank = SEA:UnitHasBuff( "player", "Burning Wish" )
				dmgM = dmgM + rank * 0.03
			end	
		elseif baseSpell.School == "Shadow" then
			if buffName == "Touch of Shadow" then
				local index, apps, texture, rank = SEA:UnitHasBuff( "player", "Touch of Shadow" )
				dmgM = dmgM + rank * 0.03
			elseif buffName =="Shadowform" then
				dmgM = dmgM + 0.15
			end		
		end	
	
		if buffName == "Amplify Curse" and spellName == "Curse of Agony" then
	 		bDmgM = bDmgM + 0.5
		elseif buffName ==  "Positive Charge" then
			local index, apps, texture, rank = SEA:UnitHasBuff( "player", "Positive Charge" )
			dmgM = dmgM + 0.1 * apps
		elseif buffName == "Negative Charge" then
			local index, apps, texture, rank = SEA:UnitHasBuff( "player", "Negative Charge" )
			dmgM = dmgM + 0.1 * apps
		elseif buffName == "Traces of Silithyst" then 
			dmgM = dmgM + 0.05
		elseif buffName == "Berserking" then
			dmgM = dmgM + 0.3
		elseif buffName == "Burning Adrenaline" then
			dmgM = dmgM + 1
			castTime = 1.5
		elseif buffName == "Soul Link" then
			dmgM = dmgM + 0.03
		elseif buffName == "Master Demonologist" then
		
			--Does this work? Does MD have a rank returned?
		
			local index, apps, texture, rank = SEA:UnitBuff( "player", "Master Demonologist" )
			
			if UnitCreatureFamily( "pet" ) then
				local petFamily = UnitCreatureFamily("pet")
				
				if petFamily == "Succubus" then
					dmgM = dmgM + 0.02 * rank
				elseif petFamily == "Felguard" then
					dmgM = dmgM + 0.01 * rank
				end
			end
		elseif buffName == "Power Infusion" then
			spellDmgM = spellDmgM + 0.2
		end
	end
	
	--Initialize variables that are calculated
	local calcAvgDmgCrit			--Calculated final damage with crit
	local calcAvgDmg			--Calculated final damage without crit
	local calcMinDmg			--Calculated dmg range: MIN
	local calcMaxDmg			--Calculated dmg range: MAX
	local calcDotDmg			--Calculated DOT portion (if applicable)
	local igniteDotDmg			--Calculated Ignite DOT Portion
	local avgIgnite				--Calculated Average Ignite
	
	
	function DmgCalc()
	
		calcAvgDmgCrit, calcAvgDmg, calcMinDmg, calcMaxDmg, calcDotDmg, igniteDotDmg, avgIgnite = 0, 0, 0, 0, 0, 0, 0
	
		if not baseSpell.sHits then
			calcMinDmg = dmgM * ( bDmgM * spell.bMin + ( spellDmg * spellDmgM ) )
			calcMaxDmg = dmgM * ( bDmgM * spell.bMax + ( spellDmg * spellDmgM ) )
		elseif not spell.hybridDotDmg then
			calcMinDmg = dmgM * ( bDmgM * spell.bMin * baseSpell.sHits + (  spellDmg * spellDmgM ) )
			calcMaxDmg = dmgM * ( bDmgM * spell.bMax * baseSpell.sHits + (  spellDmg * spellDmgM ) )
		else
			calcMinDmg = ( dmgM + bNukeDmg ) * ( bDmgM * spell.bMin + ( spellDmg * spellDmgM ) )
			calcMaxDmg = ( dmgM + bNukeDmg ) * ( bDmgM * spell.bMax + ( spellDmg * spellDmgM ) )
		end
	
		calcAvgDmg = (calcMinDmg + calcMaxDmg) / 2
	
		if baseSpell.canCrit then
			calcAvgDmgCrit = calcAvgDmg + ( critPerc / 100 ) * calcAvgDmg * critM
		else
			calcAvgDmgCrit = calcAvgDmg
			critPerc = 0;
		end
	
		if spell.hybridDotDmg then
	
			local dotFactor = 0;
			
				if baseSpell.sFactor then
					dotFactor = baseSpell.sFactor
				end
		
			calcDotDmg = dmgM * ( spell.hybridDotDmg + ( spell.eDuration / 15 ) * ( 1 - dotFactor ) * spellDmg )
			spellDmgM = spellDmgM +  ( spell.eDuration / 15 )
		end
		
		if baseSpell.Ignite then
			igniteDotDmg = igniteM * calcAvgDmgCrit
			avgIgnite = ( critPerc / 100 ) * igniteDotDmg
		end
		
		if spell.extraDotDmg then
			calcDotDmg = calcDotDmg + spell.extraDotDmg -- TODO * somethingFactor??
		end
		
		
		--Begin DPS calculations
		
		local calcDPS
		local calcDPT
	
		if baseSpell.eDot then
			calcDPS = calcAvgDmg / spell.eDuration;
		else
		
			if baseSpell.Ignite then
				calcDPS = calcAvgDmgCrit / castTime + avgIgnite / 4
				calcAvgDmgCrit = calcAvgDmgCrit + avgIgnite + calcDotDmg
			elseif spell.hybridDotDmg or spell.extraDotDmg then
				calcDPS = calcAvgDmgCrit / castTime + calcDotDmg / spell.eDuration
				calcAvgDmgCrit = calcAvgDmgCrit + calcDotDmg
			else
				calcDPS = calcAvgDmgCrit / castTime
			end
		end
		
		calcDPT = calcAvgDmgCrit / castTime
		
		return DrDamage:Round( calcMinDmg ), DrDamage:Round( calcMaxDmg ), DrDamage:Round( calcAvgDmg ), calcAvgDmgCrit, DrDamage:Round( critPerc, 2 ), DrDamage:Round( calcDotDmg ), DrDamage:Round( calcDPS ), DrDamage:Round( calcDPT ), DrDamage:Round( spellDmg ), DrDamage:Round( igniteDotDmg )
		
	end
	
	local returnMinDmg, returnMaxDmg, returnAvgDmg, returnAvgDmgCrit, returnCritPerc, returnDotDmg, returnDPS, returnDPTC, returnSpellDmg, returnIgniteDotDmg = DmgCalc()
	
	local returnNextCrit = 0
	local returnNextTenDam = 0
	
	if baseSpell.canCrit then
		critPerc = critPerc + 1
		local _, _, _, NextCrit = DmgCalc()
		returnNextCrit = DrDamage:Round( NextCrit - returnAvgDmgCrit, 1 )
	end
	
	if baseSpell.canCrit then
		critPerc = critPerc - 1
		spellDmg = spellDmg + 10
		local _,_,_, NextTen = DmgCalc()
		returnNextTenDam = DrDamage:Round( NextTen - returnAvgDmgCrit, 1 )
	else
		spellDmg = spellDmg + 10
		local _,_,_, NextTen = DmgCalc()
		returnNextTenDam = DrDamage:Round( NextTen - returnAvgDmgCrit, 1 )
	end
	
	
			
	return returnMinDmg, returnMaxDmg, returnAvgDmg, DrDamage:Round( returnAvgDmgCrit ), returnCritPerc, returnDotDmg, returnDPS, returnDPTC, returnSpellDmg, DrDamage:Round( spellDmgM, 2 ), DrDamage:Round( dmgM, 2), returnNextCrit, returnNextTenDam, returnIgniteDotDmg
end

function DrDamage:Round(x, y)

	local s
	
	if y then 
		x = x * math.pow(10, y) 
	end
	
	if (x > 0) then
		s = math.floor(x+0.5);
  	else
    		s = math.ceil(x-0.5);
  	end
  	
  	if y then 
  		s = s / math.pow(10, y) 
  	end
  	
  	return s
end

